@@ -27,17 +27,7 @@
 #ifdef _OPENMP
 #include <omp.h>
 #endif
-
-/* binary tree; dictionary of qgrams */
-
-typedef struct qnode {
-  unsigned int *qgram; // the q-gram.
-  double *n;           // (vector of) counts.
-  struct qnode *left;
-  struct qnode *right;
-} qtree;
-
-
+#include "qtree.h"
 
 
 
@@ -198,6 +188,17 @@ static void *alloc(type t){
 /* -- END OF ALLOCATOR -- */
 
 
+// helper functions
+qtree *new_qtree(int q, int nstr){
+  init_shelf(q, nstr);
+  return NULL;
+}
+
+void free_qtree(){
+  clear_shelf();
+}
+
+
 /* Lexicographical comparison of two qgrams.
  * output:
  * -1 : q1 < q2
@@ -211,16 +212,6 @@ static int compare(unsigned int *q1, unsigned int *q2, int q){
   return compare( q1 + 1, q2 + 1, q - 1 );
 }
 
-// helper functions
-static qtree *new_qtree(int q, int nstr){
-  init_shelf(q, nstr);
-  return NULL;
-}
-
-static void free_qtree(){
-  clear_shelf();
-}
-
 
 /* push qgram into binary tree
  * 
@@ -273,7 +264,6 @@ static qtree *push_string(unsigned int *str, int strlen, unsigned int q, qtree *
 
 
 
-/* The real work starts here */
 
 /* get qgram-distance from tree and set all qgram-freqencies 
  * to 0 (so the tree can be reused).
@@ -343,11 +333,11 @@ static void getjaccard(qtree *Q, double *d){
  * -1   : infinite distance
  * -2   : Not enough memory
  */
-static double qgram_tree(
+double qgram_dist(
     unsigned int *s, 
+    int x,
     unsigned int *t, 
-    unsigned int x,
-    unsigned int y,
+    int y,
     unsigned int q, 
     qtree *Q,
     int distance
@@ -394,171 +384,6 @@ static double qgram_tree(
   return dist[0];
 }
 
-/* R interface to qgram distance */
-SEXP R_qgram_tree(SEXP a, SEXP b, SEXP qq, SEXP distance, SEXP useBytes, SEXP nthrd){
-  PROTECT(a);
-  PROTECT(b);
-  PROTECT(qq);
-  PROTECT(distance);
-  PROTECT(useBytes);
-  PROTECT(nthrd);
-
-  // choose distance function
-
-  int dist = INTEGER(distance)[0]
-    , q = INTEGER(qq)[0]
-    , na = length(a)
-    , nb = length(b)
-    , nt = (na > nb) ? na : nb
-    , ml_a = max_length(a)
-    , ml_b = max_length(b)
-    , bytes = INTEGER(useBytes)[0];
-
-  // output
-  SEXP yy; 
-  PROTECT(yy = allocVector(REALSXP, nt));
-  double *y = REAL(yy);
-
-  #ifdef _OPENMP 
-  int  nthreads = INTEGER(nthrd)[0];
-  #pragma omp parallel num_threads(nthreads) default(none) \
-      shared(y, R_PosInf, NA_REAL, bytes, dist, q, na, nb, ml_a, ml_b, nt, a, b)
-  #endif
-  {
-    // set up a qtree; 
-    qtree *Q = new_qtree(q, 2L);
-    unsigned int *s = NULL, *t = NULL;
-    s = (unsigned int *) malloc( (2L + ml_a + ml_b) * sizeof(int) );
-    t = s + ml_a + 1L;
-    if ( s == NULL ) nt = -1;
-   
-    int k, len_s, len_t, isna_s, isna_t
-      , i = 0, j = 0, ID = 0, num_threads = 1;
-
-    #ifdef _OPENMP
-    ID = omp_get_thread_num();
-    num_threads = omp_get_num_threads();
-    i = recycle(ID-num_threads, num_threads, na);
-    j = recycle(ID-num_threads, num_threads, nb);
-    #endif
-
-    for ( k = ID; k < nt; k += num_threads ){ 
-
-      get_elem1(a, i, bytes, &len_s, &isna_s, s);
-      get_elem1(b, j, bytes, &len_t, &isna_t, t);
-
-      if ( isna_s || isna_t ){
-        y[k] = NA_REAL;
-      } else {
-        y[k] = qgram_tree(s, t, len_s, len_t, q, Q, dist);
-        if (y[k] == -2.0){
-          UNPROTECT(5);
-          error("Unable to allocate enough memory");
-        }
-        if (y[k] == -1.0){
-          y[k] = R_PosInf;
-        }
-      }
-      i = recycle(i, num_threads, na);
-      j = recycle(j, num_threads, nb);
-    }
-    free_qtree();
-    free(s);
-  }
-  UNPROTECT(7);
-  if (nt < 0)  error("Unable to allocate enough memory");
-  return yy;
-}
-
-
-//-- Match function interface with R
-
-SEXP R_match_qgram_tree(SEXP x, SEXP table, SEXP nomatch, SEXP matchNA, SEXP qq
-    , SEXP maxDist, SEXP distance, SEXP useBytes, SEXP nthrd){
-  PROTECT(x);
-  PROTECT(table);
-  PROTECT(nomatch);
-  PROTECT(matchNA);
-  PROTECT(qq);
-  PROTECT(maxDist);
-  PROTECT(distance);
-  PROTECT(useBytes);
-  PROTECT(nthrd);
-
-  double max_dist = REAL(maxDist)[0] == 0.0 ? R_PosInf : REAL(maxDist)[0];
-  
-  
-  int dist = INTEGER(distance)[0] // choose distance function
-    , q = INTEGER(qq)[0]
-    , nx = length(x)
-    , ntable = length(table)
-    , no_match = INTEGER(nomatch)[0]
-    , match_na = INTEGER(matchNA)[0]
-    , bytes = INTEGER(x)[0];
-  
-  // convert to integer. 
-  Stringset *X = new_stringset(x, bytes);
-  Stringset *T = new_stringset(table, bytes);
-
-  // output vector
-  SEXP yy;
-  PROTECT(yy = allocVector(INTSXP, nx));
-  int *y = INTEGER(yy);
-
-  
-  #ifdef _OPENMP
-  int nthreads = INTEGER(nthrd)[0];
-  #pragma omp parallel num_threads(nthreads) default(none) \
-    shared(X, T, y, R_PosInf,NA_INTEGER, nx, ntable, no_match, match_na, bytes, q, dist, max_dist)
-  #endif
-  {
-    // set up a qtree;
-    qtree *Q = new_qtree(q, 2);
-
-    double d = R_PosInf, d1 = R_PosInf;
-    int index, len_X, len_T;
-    unsigned int *str, **tab;
-
-    #ifdef _OPENMP
-    #pragma omp for
-    #endif
-    for ( int i=0; i<nx; i++){
-      index = no_match;
-      len_X = X->str_len[i];
-      str = X->string[i];
-      tab = T->string;
-      d1 = R_PosInf;
-      for ( int j=0; j<ntable; j++, tab++){
-        len_T = T->str_len[j];
-        if ( len_X != NA_INTEGER && len_T != NA_INTEGER ){ // both are char (usual case)
-          d = qgram_tree(
-            str, *tab, len_X, len_T, q, Q, dist
-          );
-          if ( d == -2.0 ) nx = -1;
-          if ( d > max_dist ){
-            continue;
-          } else if ( d > -1 && d < d1){ 
-            index = j + 1;
-            if ( fabs(d) < 1e-14 ) break; 
-            d1 = d;
-          }
-        } else if ( len_X == NA_INTEGER && len_T == NA_INTEGER ) {  // both are NA
-          index = match_na ? j + 1 : no_match;
-          break;
-        }
-      }
-      
-      y[i] = index;
-    } 
-    free_qtree();
-  } // end of parallel region
-  free_stringset(X);
-  free_stringset(T);
-  UNPROTECT(10);
-  if (nx < 0 ) error("Unable to allocate enough memory");
-  return(yy);
-}
-
 
 /* R interface to qgram tabulator */
 
